#include "box.h"

bool Box::
any_intersection(const Ray& ray,
                 float smin,
                 float smax) const
{
   bool intersected=false;
   for(int axis=0; axis<3; ++axis){
      if(ray.direction[axis]==0){ // is ray parallel to this slab?
         // if the ray doesn't start in the slab it's parallel to,
         // it can't intersect the box
         if(ray.origin[axis]<box.xmin[axis] || ray.origin[axis]>box.xmax[axis])
            return false;
      }else{ // if not, we can look for intersections
         float s0=-(ray.origin[axis]-box.xmin[axis])/ray.direction[axis],
               s1=-(ray.origin[axis]-box.xmax[axis])/ray.direction[axis];
         if(s1<s0) std::swap(s0,s1);
         if(s0>smin){
            smin=s0;
            intersected=true;
         }
         if(s1<smax){
            smax=s1;
            intersected=true;
         }
         if(smin>smax) return false; // if interval empty, no intersection
      }
   }
   return intersected;
}

bool Box::
first_intersection(const Ray& ray,
                   float smin,
                   float smax,
                   float& s,
                   Vec3f& normal) const
{
   // loop over axes, intersecting with slab, saving result
   int face=-1;
   bool entering=false;
   float closest_s=smax+1;
   for(int axis=0; axis<3; ++axis){
      if(ray.direction[axis]==0){ // is ray parallel to this slab?
         // if the ray doesn't start in the slab it's parallel to,
         // it can't intersect the box
         if(ray.origin[axis]<box.xmin[axis] || ray.origin[axis]>box.xmax[axis])
            return false;
      }else{ // if not, we can look for intersections
         float s0=-(ray.origin[axis]-box.xmin[axis])/ray.direction[axis],
               s1=-(ray.origin[axis]-box.xmax[axis])/ray.direction[axis];
         if(s0<=s1){
            if(s0>=smin){
               smin=s0;
               face=2*axis;
               entering=true;
               closest_s=s0;
            }
            if(s1<=smax){
               smax=s1;
               if(!entering){ // no entry detected yet?
                  face=2*axis+1;
                  closest_s=s1;
               }
            }
         }else{
            if(s1>=smin){
               smin=s1;
               face=2*axis+1;
               entering=true;
               closest_s=s1;
            }
            if(s0<=smax){
               smax=s0;
               if(!entering){ // no entry detected yet?
                  face=2*axis;
                  closest_s=s0;
               }
            }
         }
         if(smin>smax) return false; // if interval empty, no intersection
      }
   }
   switch(face){
      case -1: return false;
      case 0: normal[0]=-1; normal[1]= 0; normal[2]= 0; break;
      case 1: normal[0]= 1; normal[1]= 0; normal[2]= 0; break;
      case 2: normal[0]= 0; normal[1]=-1; normal[2]= 0; break;
      case 3: normal[0]= 0; normal[1]= 1; normal[2]= 0; break;
      case 4: normal[0]= 0; normal[1]= 0; normal[2]=-1; break;
      case 5: normal[0]= 0; normal[1]= 0; normal[2]= 1; break;
   }
   s=closest_s;
   return true;
}
