#include <iostream>
#include <fstream>
#include "bfstream.h"
#include "camera.h"
#include "image_io.h"
#include "mesh_io.h"
#include "box.h"
#include "plane.h"
#include "sphere.h"
#include "triangle_mesh.h"
#include "diffuse_shader.h"
#include "emission_shader.h"
#include "normal_shader.h"
#include "mirror_shader.h"

int main(int argc, char** argv)
{
	// set up lights
	Lighting lighting;
	lighting.ambient=Vec3f(0.1,0.1,0.1);
	lighting.directed_light.push_back(new PointLight(Vec3f(1, 3, 2),
			10.f*Vec3f(0.8,0.9,1)));
	lighting.directed_light.push_back(new PointLight(Vec3f(0, 3.5f, 8),
			0.5f*Vec3f(1,0.8,0.6)));

	MultiThing world;
	// add the ground
	DiffuseShader ground_shader(Vec3f(1,1,1));
	Plane ground_shape;
	StandardThing ground(ground_shape, ground_shader);
	world.subthing.push_back(&ground);
	// add a couple of spheres
	MirrorShader sphere1_shader(Vec3f(0.7, 0.7, 0.7));
	Sphere sphere1_shape(Vec3f(0.75, 1.7, 2), 0.7);
	StandardThing sphere1(sphere1_shape, sphere1_shader);
	world.subthing.push_back(&sphere1);
	MirrorShader sphere2_shader(Vec3f(1, 1, 0.9));
	Sphere sphere2_shape(Vec3f(-0.75, 2.1, 0), 0.5);
	StandardThing sphere2(sphere2_shape, sphere2_shader);
	world.subthing.push_back(&sphere2);
	EmissionShader sphere3_shader(10.f*Vec3f(0.8,0.9,1));
	Sphere sphere3_shape(Vec3f(1, 3, 2), 0.2);
	StandardThing sphere3(sphere3_shape, sphere3_shader);
	world.subthing.push_back(&sphere3);
	// add a triangle mesh object
	DiffuseShader mesh_shader(Vec3f(0.8, 0.4, 0.2));
	TriangleMesh mesh_shape;
	read_objfile(mesh_shape.tri, mesh_shape.x, 0, "example.obj");
	for(unsigned int i=0; i<mesh_shape.x.size(); ++i)
		mesh_shape.x[i][1]+=1;
	mesh_shape.build_tree();
	StandardThing mesh_thing(mesh_shape, mesh_shader);
	world.subthing.push_back(&mesh_thing);
	// put a box in
	DiffuseShader box1_shader(Vec3f(1, 0.6, 0.5));
	Box box1_shape(Vec3f(2, 0.1f, -3), Vec3f(3.5f, 1.1f, -1.5f));
	StandardThing box1(box1_shape, box1_shader);
	world.subthing.push_back(&box1);
	// finalize
	world.build_tree();

	// set up raytracer
	// (this has to come after lighting and world, since raytracer needs a reference to these objects)
	RayTracer raytracer(lighting, world, 1e-4, Vec3f(0,0,0));

	// and finally set up the camera
	// (this has to come after raytracer, since the camera needs a reference to it)
	Camera camera(raytracer);
	camera.modelview=look_at(Vec3f(0, 3.5f, 8), Vec3f(0, 1, 0));

	// do the render
	Array2<Vec3f> image;
	camera.render_image(image, 2);

	// and save the output image
	write_sgi(image, "output.sgi");

	return 0;
}

