#include "plane.h"

bool Plane::
any_intersection(const Ray& ray,
                 float smin,
                 float smax) const
{
   float dn=dot(ray.direction, plane_normal);
   if(dn==0) return false;
   float s=-dot(ray.origin-point,plane_normal)/dn;
   return (s>=smin && s<=smax);
}

bool Plane::
first_intersection(const Ray& ray,
                   float smin,
                   float smax,
                   float& s,
                   Vec3f& normal) const
{
   float dn=dot(ray.direction, plane_normal);
   if(dn==0) return false;
   float s0=-dot(ray.origin-point,plane_normal)/dn;
   if(s0>=smin && s0<=smax){
      s=s0;
      normal=plane_normal;
      return true;
   }else
      return false;
}

bool Plane::
first_intersection(const Ray& ray,
                   float smin,
                   float smax,
                   float& s) const
{
   float dn=dot(ray.direction, plane_normal);
   if(dn==0) return false;
   float s0=-dot(ray.origin-point,plane_normal)/dn;
   if(s0>=smin && s0<=smax){
      s=s0;
      return true;
   }else
      return false;
}

BoundingBox Plane::
bounding_box(void) const
{
   // the max call is there to prevent a completely empty bounding box from
   // being created for a plane which is parallel to one of the axes. That
   // could cause the primitive bounding box intersection test to fail, which
   // would be bad.
   Vec3f vector_bound(bound*max(std::sqrt((1-sqr(plane_normal[0]))),1e-7f),
                      bound*max(std::sqrt((1-sqr(plane_normal[1]))),1e-7f),
                      bound*max(std::sqrt((1-sqr(plane_normal[2]))),1e-7f));
   return BoundingBox(point-vector_bound, point+vector_bound);
}
