#ifndef PLANE_H
#define PLANE_H

#include "shape.h"

// a (nearly) infinite plane
struct Plane: public Shape
{
   Vec3f point, plane_normal;
   float bound; // to make planes work with bounding boxes, we add a bound
                // (distance from point)

   Plane(void) : point(0,0,0), plane_normal(0,1,0), bound(1e4) {}

   Plane(const Vec3f& point_,
         const Vec3f& plane_normal_,
         float bound_)
      : point(point_),
        plane_normal(plane_normal_),
        bound(bound_)
   {}

   virtual bool
   any_intersection(const Ray& ray,
                    float smin,
                    float smax) const;

   virtual bool
   first_intersection(const Ray& ray,
                      float smin,
                      float smax,
                      float& s,
                      Vec3f& normal) const;
   
   virtual bool
   first_intersection(const Ray& ray,
                      float smin,
                      float smax,
                      float& s) const;

   virtual BoundingBox
   bounding_box(void) const;

   virtual bool
   contains(const Vec3f& x) const
   { return dot(x-point, plane_normal)<=0; }
};

#endif
