#ifndef RAYTRACER_H
#define RAYTRACER_H

#include <limits>
#include "lighting.h"
#include "thing.h"

struct RayTracer
{
   const Lighting& lighting;
   const Thing& thing;
   float epsilon;
   Vec3f background_color;
   int initial_max_depth;

   RayTracer(const Lighting& lighting_,
             const Thing& thing_,
             float epsilon_,
             Vec3f background_color_=Vec3f(0,0,0),
             int initial_max_depth_=8)
      : lighting(lighting_),
        thing(thing_),
        epsilon(epsilon_),
        background_color(background_color_),
        initial_max_depth(initial_max_depth_)
   {}

   // gather light from a point on a surface
   Vec3f
   gather_light(const Vec3f& x,
                const Vec3f& normal) const;

   // return light calculated along the given ray
   Vec3f
   trace_ray(const Ray& ray) const
   {
      float s;
      return trace_ray(ray, epsilon, std::numeric_limits<float>::max(), s,
                       initial_max_depth);
   }

   // return light calculated along the given ray
   Vec3f
   trace_ray(const Ray& ray,
             float smin,
             float smax,
             float &s,
             int max_depth) const
   {
      // base case
      if(max_depth<=0) 
      {
         s = smax;
         return lighting.ambient;
      }
      
      // otherwise trace a ray
      Vec3f normal;
      const Shader* shader=thing.intersect(ray, smin, smax, s, normal);
      if(shader){ // if we hit something
         Vec3f c=shader->evaluate(ray, s, normal, *this, max_depth-1);
         return c;
      }else
         return background_color;
   }
};

#endif
