#ifndef SHAPE_H
#define SHAPE_H

#include <limits>
#include "array1.h"
#include "primitives.h"
#include "bounding_box_tree.h"

struct Shape
{
   virtual
   ~Shape(void)
   {}

   virtual bool
   any_intersection(const Ray& ray,
                    float smin,
                    float smax) const
   { // default implementation
      float s;
      Vec3f normal;
      return first_intersection(ray, smin, smax, s, normal);
   }

   // if no intersection, does not modify s or normal!
   virtual bool
   first_intersection(const Ray& ray,
                      float smin,
                      float smax,
                      float& s,
                      Vec3f& normal) const = 0;

   // if no intersection, does not modify s!
   virtual bool
   first_intersection(const Ray& ray,
                      float smin,
                      float smax,
                      float& s) const
   { // default implementation
      Vec3f normal;
      return first_intersection(ray, smin, smax, s, normal);
   }

   virtual BoundingBox
   bounding_box(void) const = 0;

   virtual bool
   contains(const Vec3f& x) const
   {
      if(!bounding_box().contains(x))
         return false;
      Ray test(x, Vec3f(0.31746083338432495857f,
                        0.79467229247405857251f,
                        0.51741140965478726876f));
      float s;
      Vec3f normal;
      if(first_intersection(test, 0, std::numeric_limits<float>::max(),
                            s, normal))
         return dot(normal, test.direction)>0;
      else
         return false;
   }
};

//==============================================================================

struct MultiShape: public Shape
{
   Array1<Shape*> subshape;
   BoundingBoxTree tree;

   // need to call this before intersections or boxes, after subshape is filled
   void
   build_tree(void);

   virtual bool
   any_intersection(const Ray& ray,
                    float smin,
                    float smax) const;

   virtual bool
   first_intersection(const Ray& ray,
                      float smin,
                      float smax,
                      float& s,
                      Vec3f& normal) const;

   virtual bool
   first_intersection(const Ray& ray,
                      float smin,
                      float smax,
                      float& s) const;

   virtual BoundingBox
   bounding_box(void) const
   { return tree.box; }

   virtual bool
   contains(const Vec3f& x) const;
};

#endif
