#include "thing.h"

void MultiThing::
build_tree(void)
{
   std::vector<BoundingBox> box(subthing.size());
   for(unsigned int i=0; i<subthing.size(); ++i)
      box[i]=subthing[i]->bounding_box();
   tree.construct_from_leaf_boxes(box.size(), &box[0]);
}

Vec3f MultiThing::
shadow_transmittance(const Ray& ray,
                     float smin,
                     float smax) const
{
   Vec3f transmittance(1,1,1);
   // first check if the ray hits the root's bounding box
   if(!tree.box.intersect_and_update(ray, smin, smax))
      return transmittance;
   std::vector<const BoundingBoxTree*> stack;
   stack.push_back(&tree);
   while(!stack.empty()){
      const BoundingBoxTree *node=stack.back();
      stack.pop_back();
      // check if there's any geometry to check in this node
      for(unsigned int i=0; i<node->index.size(); ++i)
         transmittance*=subthing[node->index[i]]->shadow_transmittance(
                                                              ray, smin, smax);
      // check for children nodes (with boxes intersecting the ray)
      for(unsigned int i=0; i<node->children.size(); ++i){
         if(node->children[i]->box.intersect(ray, smin, smax))
            stack.push_back(node->children[i]); 
      }
   }
   return transmittance;
}

const Shader* MultiThing::
intersect(const Ray& ray,
          float smin,
          float smax,
          float& s,
          Vec3f& normal) const
{
   // first check if the ray hits the root's bounding box
   if(!tree.box.intersect_and_update(ray, smin, smax))
      return false;
   const Shader* shader=0;
   std::vector<const BoundingBoxTree*> stack; // ideally use a heap instead
   stack.push_back(&tree);
   while(!stack.empty()){
      const BoundingBoxTree *node=stack.back();
      stack.pop_back();
      // check if there's any geometry to check in this node
      for(unsigned int i=0; i<node->index.size(); ++i){
         const Shader* shader_test=subthing[node->index[i]]->intersect(
                                                   ray, smin, smax, s, normal);
         if(shader_test){
            shader=shader_test;
            smax=s; // eliminate any more distant intersections
         }
      }
      // children nodes to check?
      for(unsigned int i=0; i<node->children.size(); ++i){
         if(node->children[i]->box.intersect(ray, smin, smax))
            stack.push_back(node->children[i]); 
      }
   }
   return shader;
}

BoundingBox MultiThing::
bounding_box(void) const
{
   return tree.box;
}
