#ifndef THING_H
#define THING_H

#include "array1.h"
#include "primitives.h"
#include "shape.h"
#include "shader.h"
#include "bounding_box_tree.h"

struct RayTracer;

struct Thing
{
   virtual
   ~Thing(void)
   {}

   virtual Vec3f
   shadow_transmittance(const Ray& ray,
                        float smin,
                        float smax) const = 0;

   // return 0 if no intersection (and DO NOT MODIFY s!)
   // otherwise set s for intersection and return pointer to shader.
   virtual const Shader*
   intersect(const Ray& ray,
             float smin,
             float smax,
             float& s,
             Vec3f& normal) const = 0;

   virtual BoundingBox
   bounding_box(void) const = 0;
};

//=============================================================================

struct StandardThing: public Thing
{
   const Shape& shape;
   const Shader& shader;

   StandardThing(const Shape& shape_,
                 const Shader& shader_)
      : shape(shape_), shader(shader_)
   {}

   virtual Vec3f
   shadow_transmittance(const Ray& ray,
                        float smin,
                        float smax) const
   {
      return shader.shadow_transmittance(shape, ray, smin, smax);
   }

   virtual const Shader*
   intersect(const Ray& ray,
             float smin,
             float smax,
             float& s,
             Vec3f& normal) const
   {
      if(shape.first_intersection(ray, smin, smax, s, normal))
         return &shader;
      else
         return 0;
   }

   virtual BoundingBox
   bounding_box(void) const
   {
      return shape.bounding_box();
   }
};

//=============================================================================

struct MultiThing: public Thing
{
   Array1<Thing*> subthing;
   BoundingBoxTree tree;

   // need to call this before intersections or boxes, after subthing is filled
   void
   build_tree(void);

   virtual Vec3f
   shadow_transmittance(const Ray& ray,
                        float smin,
                        float smax) const;

   virtual const Shader*
   intersect(const Ray& ray,
             float smin,
             float smax,
             float& s,
             Vec3f& normal) const;

   virtual BoundingBox
   bounding_box(void) const;
};

#endif
