#ifndef TRIANGLE_MESH_H
#define TRIANGLE_MESH_H

#include "shape.h"
#include "bounding_box_tree.h"
// a mesh of triangles
struct TriangleMesh: public Shape
{
   std::vector<Vec3i> tri; // an array of vertex indices, three per triangle
   std::vector<Vec3f> x; // an array of vertex positions
   BoundingBoxTree tree; // a hierarchy to speed up intersection tests

   // Must be called after setting up tri and x, before any geometry is tested.
   void
   build_tree(void);

   virtual bool
   any_intersection(const Ray& ray,
                    float smin,
                    float smax) const;

   virtual bool
   first_intersection(const Ray& ray,
                      float smin,
                      float smax,
                      float& s,
                      Vec3f& normal) const;

   virtual BoundingBox
   bounding_box(void) const;

   protected:
   bool
   intersect_triangle(unsigned int f,
                      const Ray& ray,
                      float smin,
                      float smax,
                      float& s,
                      float& alpha,
                      float& beta,
                      float& gamma) const;
};

#endif
